@@ -241,16 +241,8 @@ void LLVMToSPIRVDbgTran::transLocationInfo() {
               V = VPrev;
             }
           }
-          if (BM->getDebugInfoEIS() ==
-                  SPIRVEIS_NonSemantic_Shader_DebugInfo_100 ||
-              BM->getDebugInfoEIS() ==
-                  SPIRVEIS_NonSemantic_Shader_DebugInfo_200)
-            BM->addDebugLine(V, getVoidTy(),
-                             File ? File->getId() : getDebugInfoNoneId(),
-                             LineNo, LineNo, Col, Col + 1);
-          else
-            BM->addLine(V, File ? File->getId() : getDebugInfoNoneId(), LineNo,
-                        Col);
+          BM->addLine(V, File ? File->getId() : getDebugInfoNone()->getId(),
+                      LineNo, Col);
         }
       } // Instructions
     }   // Basic Blocks
@@ -268,17 +260,6 @@ SPIRVEntry *LLVMToSPIRVDbgTran::transDbgEntry(const MDNode *DIEntry) {
   }
   SPIRVEntry *Res = transDbgEntryImpl(DIEntry);
   assert(Res && "Translation failure");
-  // We might end up having a recursive debug info generation like the
-  // following:
-  // translation of DIDerivedType (member) calls DICompositeType translation
-  // as its parent scope;
-  // translation of DICompositeType calls translation of its members
-  // (DIDerivedType with member tag).
-  // Here we make only the latest of these instructions be cached and hence
-  // reused
-  // FIXME: find a way to not create dead instruction
-  if (MDMap[DIEntry])
-    return MDMap[DIEntry];
   MDMap[DIEntry] = Res;
   return Res;
 }
@@ -460,7 +441,7 @@ void LLVMToSPIRVDbgTran::transformToConstant(std::vector<SPIRVWord> &Ops,
   }
 }
 
-SPIRVWord LLVMToSPIRVDbgTran::mapDebugFlags(DINode::DIFlags DFlags) {
+SPIRVWord mapDebugFlags(DINode::DIFlags DFlags) {
   SPIRVWord Flags = 0;
   if ((DFlags & DINode::FlagAccessibility) == DINode::FlagPublic)
     Flags |= SPIRVDebug::FlagIsPublic;
@@ -490,13 +471,12 @@ SPIRVWord LLVMToSPIRVDbgTran::mapDebugFlags(DINode::DIFlags DFlags) {
     Flags |= SPIRVDebug::FlagTypePassByValue;
   if (DFlags & DINode::FlagTypePassByReference)
     Flags |= SPIRVDebug::FlagTypePassByReference;
-  if (BM->getDebugInfoEIS() == SPIRVEIS_NonSemantic_Shader_DebugInfo_200)
-    if (DFlags & DINode::FlagBitField)
-      Flags |= SPIRVDebug::FlagBitField;
+  if (DFlags & DINode::FlagEnumClass)
+    Flags |= SPIRVDebug::FlagIsEnumClass;
   return Flags;
 }
 
-SPIRVWord LLVMToSPIRVDbgTran::transDebugFlags(const DINode *DN) {
+SPIRVWord transDebugFlags(const DINode *DN) {
   SPIRVWord Flags = 0;
   if (const DIGlobalVariable *GV = dyn_cast<DIGlobalVariable>(DN)) {
     if (GV->isLocalToUnit())
@@ -568,14 +548,10 @@ SPIRVId LLVMToSPIRVDbgTran::getDebugInfoNoneId() {
 
 SPIRVEntry *LLVMToSPIRVDbgTran::transDbgCompileUnit(const DICompileUnit *CU) {
   using namespace SPIRVDebug::Operand::CompilationUnit;
-  SPIRVWordVec Ops(MinOperandCount);
+  SPIRVWordVec Ops(OperandCount);
   Ops[SPIRVDebugInfoVersionIdx] = SPIRVDebug::DebugInfoVersion;
   Ops[DWARFVersionIdx] = M->getDwarfVersion();
   Ops[SourceIdx] = getSource(CU)->getId();
-
-  if (isNonSemanticDebugInfo())
-    generateBuildIdentifierAndStoragePath(CU);
-
   auto DwarfLang =
       static_cast<llvm::dwarf::SourceLanguage>(CU->getSourceLanguage());
   Ops[LanguageIdx] =
@@ -585,17 +561,7 @@ SPIRVEntry *LLVMToSPIRVDbgTran::transDbgCompileUnit(const DICompileUnit *CU) {
   if (isNonSemanticDebugInfo())
     transformToConstant(
         Ops, {SPIRVDebugInfoVersionIdx, DWARFVersionIdx, LanguageIdx});
-
-  if (isNonSemanticDebugInfo()) {
-    if (BM->getDebugInfoEIS() == SPIRVEIS_NonSemantic_Shader_DebugInfo_200) {
-      Ops.push_back(BM->getString(CU->getProducer().str())->getId());
-    }
-  } else {
-    // TODO: Remove this workaround once we switch to NonSemantic.Shader.* debug
-    // info by default
-    BM->addModuleProcessed(SPIRVDebug::ProducerPrefix +
-                           CU->getProducer().str());
-  }
+  BM->addModuleProcessed(SPIRVDebug::ProducerPrefix + CU->getProducer().str());
   // Cache CU in a member.
   SPIRVCUMap[CU] = static_cast<SPIRVExtInst *>(
       BM->addDebugInfo(SPIRVDebug::CompilationUnit, getVoidTy(), Ops));
@@ -606,24 +572,16 @@ SPIRVEntry *LLVMToSPIRVDbgTran::transDbgCompileUnit(const DICompileUnit *CU) {
 
 SPIRVEntry *LLVMToSPIRVDbgTran::transDbgBaseType(const DIBasicType *BT) {
   using namespace SPIRVDebug::Operand::TypeBasic;
-  SPIRVWordVec Ops(OperandCountOCL);
+  SPIRVWordVec Ops(OperandCount);
   Ops[NameIdx] = BM->getString(BT->getName().str())->getId();
   ConstantInt *Size = getUInt(M, BT->getSizeInBits());
   Ops[SizeIdx] = SPIRVWriter->transValue(Size, nullptr)->getId();
   auto Encoding = static_cast<dwarf::TypeKind>(BT->getEncoding());
   SPIRVDebug::EncodingTag EncTag = SPIRVDebug::Unspecified;
   SPIRV::DbgEncodingMap::find(Encoding, &EncTag);
-  // Unset encoding if it's complex and NonSemantic.Shader.DebugInfo.200 is not
-  // enabled
-  if (EncTag == SPIRVDebug::Complex &&
-      BM->getDebugInfoEIS() != SPIRVEIS_NonSemantic_Shader_DebugInfo_200)
-    EncTag = SPIRVDebug::Unspecified;
   Ops[EncodingIdx] = EncTag;
-  if (isNonSemanticDebugInfo()) {
+  if (isNonSemanticDebugInfo())
     transformToConstant(Ops, {EncodingIdx});
-    // Flags value could not be generated by clang or by LLVM environment.
-    Ops.push_back(getDebugInfoNoneId());
-  }
   return BM->addDebugInfo(SPIRVDebug::TypeBasic, getVoidTy(), Ops);
 }
 
@@ -633,9 +591,9 @@ SPIRVEntry *LLVMToSPIRVDbgTran::transDbgPointerType(const DIDerivedType *PT) {
   SPIRVEntry *Base = transDbgEntry(PT->getBaseType());
   Ops[BaseTypeIdx] = Base->getId();
   Ops[StorageClassIdx] = ~0U; // all ones denote no address space
-  Optional<unsigned> AS = PT->getDWARFAddressSpace();
-  if (AS.hasValue()) {
-    SPIRAddressSpace SPIRAS = static_cast<SPIRAddressSpace>(AS.getValue());
+  std::optional<unsigned> AS = PT->getDWARFAddressSpace();
+  if (AS.has_value()) {
+    SPIRAddressSpace SPIRAS = static_cast<SPIRAddressSpace>(AS.value());
     Ops[StorageClassIdx] = SPIRSPIRVAddrSpaceMap::map(SPIRAS);
   }
   Ops[FlagsIdx] = transDebugFlags(PT);
@@ -728,8 +686,6 @@ LLVMToSPIRVDbgTran::transDbgArrayTypeNonSemantic(const DICompositeType *AT) {
     if (AT->isVector()) {
       assert(N == 1 && "Multidimensional vector is not expected!");
       Ops[ComponentCountIdx] = static_cast<SPIRVWord>(Count->getZExtValue());
-      if (isNonSemanticDebugInfo())
-        transformToConstant(Ops, {ComponentCountIdx});
       return BM->addDebugInfo(SPIRVDebug::TypeVector, getVoidTy(), Ops);
     }
     Ops[SubrangesIdx + I] = transDbgEntry(SR)->getId();
@@ -968,7 +924,7 @@ LLVMToSPIRVDbgTran::transDbgCompositeType(const DICompositeType *CT) {
   }
 
   if (isNonSemanticDebugInfo())
-    transformToConstant(Ops, {TagIdx, LineIdx, ColumnIdx, FlagsIdx});
+    transformToConstant(Ops, {LineIdx, ColumnIdx, FlagsIdx});
   SPIRVEntry *Res =
       BM->addDebugInfo(SPIRVDebug::TypeComposite, getVoidTy(), Ops);
 
@@ -988,14 +944,7 @@ LLVMToSPIRVDbgTran::transDbgCompositeType(const DICompositeType *CT) {
 }
 
 SPIRVEntry *LLVMToSPIRVDbgTran::transDbgMemberType(const DIDerivedType *MT) {
-  if (isNonSemanticDebugInfo())
-    return transDbgMemberTypeNonSemantic(MT);
-  return transDbgMemberTypeOpenCL(MT);
-}
-
-SPIRVEntry *
-LLVMToSPIRVDbgTran::transDbgMemberTypeOpenCL(const DIDerivedType *MT) {
-  using namespace SPIRVDebug::Operand::TypeMember::OpenCL;
+  using namespace SPIRVDebug::Operand::TypeMember;
   SPIRVWordVec Ops(MinOperandCount);
 
   Ops[NameIdx] = BM->getString(MT->getName().str())->getId();
@@ -1022,34 +971,6 @@ LLVMToSPIRVDbgTran::transDbgMemberTypeOpenCL(const DIDerivedType *MT) {
   return BM->addDebugInfo(SPIRVDebug::TypeMember, getVoidTy(), Ops);
 }
 
-SPIRVEntry *
-LLVMToSPIRVDbgTran::transDbgMemberTypeNonSemantic(const DIDerivedType *MT) {
-  using namespace SPIRVDebug::Operand::TypeMember::NonSemantic;
-  SPIRVWordVec Ops(MinOperandCount);
-
-  Ops[NameIdx] = BM->getString(MT->getName().str())->getId();
-  Ops[TypeIdx] = transDbgEntry(MT->getBaseType())->getId();
-  Ops[SourceIdx] = getSource(MT)->getId();
-  Ops[LineIdx] = MT->getLine();
-  Ops[ColumnIdx] = 0; // This version of DIDerivedType has no column number
-  ConstantInt *Offset = getUInt(M, MT->getOffsetInBits());
-  Ops[OffsetIdx] = SPIRVWriter->transValue(Offset, nullptr)->getId();
-  ConstantInt *Size = getUInt(M, MT->getSizeInBits());
-  Ops[SizeIdx] = SPIRVWriter->transValue(Size, nullptr)->getId();
-  Ops[FlagsIdx] = adjustAccessFlags(MT->getScope(), transDebugFlags(MT));
-  transDbgEntry(MT->getScope())->getId();
-  if (MT->isStaticMember()) {
-    if (llvm::Constant *C = MT->getConstant()) {
-      SPIRVValue *Val = SPIRVWriter->transValue(C, nullptr);
-      assert(isConstantOpCode(Val->getOpCode()) &&
-             "LLVM constant must be translated to SPIRV constant");
-      Ops.push_back(Val->getId());
-    }
-  }
-  transformToConstant(Ops, {LineIdx, ColumnIdx, FlagsIdx});
-  return BM->addDebugInfo(SPIRVDebug::TypeMember, getVoidTy(), Ops);
-}
-
 SPIRVEntry *LLVMToSPIRVDbgTran::transDbgInheritance(const DIDerivedType *DT) {
   using namespace SPIRVDebug::Operand::TypeInheritance;
   unsigned ParentIdx, OffsetIdx, SizeIdx, FlagsIdx, OperandCount;
@@ -1111,14 +1032,8 @@ LLVMToSPIRVDbgTran::transDbgTemplateParameter(const DITemplateParameter *TP) {
   Ops[ValueIdx] = getDebugInfoNoneId();
   if (TP->getTag() == dwarf::DW_TAG_template_value_parameter) {
     const DITemplateValueParameter *TVP = cast<DITemplateValueParameter>(TP);
-    if (auto *TVVal = TVP->getValue()) {
-      Constant *C = cast<ConstantAsMetadata>(TVVal)->getValue();
-      Ops[ValueIdx] = SPIRVWriter->transValue(C, nullptr)->getId();
-    } else {
-      SPIRVType *TyPtr = SPIRVWriter->transType(
-          PointerType::get(Type::getInt8Ty(M->getContext()), 0));
-      Ops[ValueIdx] = BM->addNullConstant(TyPtr)->getId();
-    }
+    Constant *C = cast<ConstantAsMetadata>(TVP->getValue())->getValue();
+    Ops[ValueIdx] = SPIRVWriter->transValue(C, nullptr)->getId();
   }
   Ops[SourceIdx] = getDebugInfoNoneId();
   Ops[LineIdx] = 0;   // This version of DITemplateParameter has no line number
@@ -1231,8 +1146,6 @@ SPIRVEntry *LLVMToSPIRVDbgTran::transDbgFunction(const DISubprogram *Func) {
     transformToConstant(Ops, {LineIdx, ColumnIdx, FlagsIdx});
 
   SPIRVEntry *DebugFunc = nullptr;
-  SPIRVValue *FuncDef = nullptr;
-  bool IsEntryPointKernel = false;
   if (!Func->isDefinition()) {
     DebugFunc =
         BM->addDebugInfo(SPIRVDebug::FunctionDeclaration, getVoidTy(), Ops);
@@ -1247,36 +1160,12 @@ SPIRVEntry *LLVMToSPIRVDbgTran::transDbgFunction(const DISubprogram *Func) {
     Ops[FunctionIdIdx] = getDebugInfoNoneId();
     for (const llvm::Function &F : M->functions()) {
       if (Func->describes(&F)) {
-        // Function definition of spir_kernel can have no "spir_kernel" calling
-        // convention because SPIRVRegularizeLLVMBase::addKernelEntryPoint pass
-        // could have turned it to spir_func. The "true" entry point is a
-        // wrapper kernel function, which can be found further in the module.
-        if (FuncDef) {
-          if (F.getCallingConv() == CallingConv::SPIR_KERNEL) {
-            IsEntryPointKernel = true;
-            break;
-          }
-          continue;
-        }
-
         SPIRVValue *SPIRVFunc = SPIRVWriter->getTranslatedValue(&F);
         assert(SPIRVFunc && "All function must be already translated");
         Ops[FunctionIdIdx] = SPIRVFunc->getId();
-        FuncDef = SPIRVFunc;
-        if (!isNonSemanticDebugInfo())
-          break;
-
-        // Most likely unreachable because of Regularise LLVM pass
-        if (F.getCallingConv() == CallingConv::SPIR_KERNEL) {
-          IsEntryPointKernel = true;
-          break;
-        }
+        break;
       }
     }
-    // For NonSemantic.Shader.DebugInfo we store Function Id index as a
-    // separate DebugFunctionDefinition instruction.
-    if (isNonSemanticDebugInfo())
-      Ops.pop_back();
 
     if (DISubprogram *FuncDecl = Func->getDeclaration())
       Ops.push_back(transDbgEntry(FuncDecl)->getId());
@@ -1303,56 +1192,9 @@ SPIRVEntry *LLVMToSPIRVDbgTran::transDbgFunction(const DISubprogram *Func) {
   if (DITemplateParameterArray TPA = Func->getTemplateParams()) {
     DebugFunc = transDbgTemplateParams(TPA, DebugFunc);
   }
-
-  if (isNonSemanticDebugInfo() &&
-      (Func->isMainSubprogram() || IsEntryPointKernel)) [[maybe_unused]]
-    SPIRVEntry *Inst = transDbgEntryPoint(Func, DebugFunc);
-
-  if (isNonSemanticDebugInfo() && FuncDef) [[maybe_unused]]
-    SPIRVEntry *Inst = transDbgFuncDefinition(FuncDef, DebugFunc);
-
   return DebugFunc;
 }
 
-SPIRVEntry *LLVMToSPIRVDbgTran::transDbgFuncDefinition(SPIRVValue *FuncDef,
-                                                       SPIRVEntry *DbgFunc) {
-  using namespace SPIRVDebug::Operand::FunctionDefinition;
-  SPIRVWordVec Ops(OperandCount);
-  Ops[FunctionIdx] = DbgFunc->getId();
-  Ops[DefinitionIdx] = FuncDef->getId();
-  SPIRVFunction *F = static_cast<SPIRVFunction *>(FuncDef);
-  SPIRVBasicBlock *BB = F->getNumBasicBlock() ? F->getBasicBlock(0) : nullptr;
-  SPIRVId ExtSetId = BM->getExtInstSetId(BM->getDebugInfoEIS());
-
-  return BM->addExtInst(getVoidTy(), ExtSetId, SPIRVDebug::FunctionDefinition,
-                        Ops, BB, BB->getInst(0));
-}
-
-SPIRVEntry *LLVMToSPIRVDbgTran::transDbgEntryPoint(const DISubprogram *Func,
-                                                   SPIRVEntry *DbgFunc) {
-  using namespace SPIRVDebug::Operand::EntryPoint;
-  SPIRVWordVec Ops(OperandCount);
-  Ops[EntryPointIdx] = DbgFunc->getId();
-
-  DICompileUnit *CU = Func->getUnit();
-  if (!CU) {
-    Ops[CompilationUnitIdx] = SPIRVCUMap.begin()->second->getId();
-    SPIRVWord EmptyStrIdx = BM->getString("")->getId();
-    Ops[CompilerSignatureIdx] = EmptyStrIdx;
-    Ops[CommandLineArgsIdx] = EmptyStrIdx;
-    return BM->addDebugInfo(SPIRVDebug::EntryPoint, getVoidTy(), Ops);
-  }
-
-  StringRef Producer = CU->getProducer();
-  StringRef Flags = CU->getFlags();
-  SPIRVEntry *CUVal = SPIRVCUMap[CU] ? SPIRVCUMap[CU] : getDebugInfoNone();
-
-  Ops[CompilationUnitIdx] = CUVal->getId();
-  Ops[CompilerSignatureIdx] = BM->getString(Producer.str())->getId();
-  Ops[CommandLineArgsIdx] = BM->getString(Flags.str())->getId();
-  return BM->addDebugInfo(SPIRVDebug::EntryPoint, getVoidTy(), Ops);
-}
-
 // Location information
 
 SPIRVEntry *LLVMToSPIRVDbgTran::transDbgScope(const DIScope *S) {
@@ -1378,12 +1220,6 @@ SPIRVEntry *LLVMToSPIRVDbgTran::transDbgScope(const DIScope *S) {
     Ops[LineIdx] = 0;   // This version of DINamespace has no line number
     Ops[ColumnIdx] = 0; // This version of DINamespace has no column number
     Ops.push_back(BM->getString(NS->getName().str())->getId());
-    if (BM->getDebugInfoEIS() == SPIRVEIS_NonSemantic_Shader_DebugInfo_200) {
-      SPIRVValue *ExpConst = BM->addConstant(
-          SPIRVWriter->transType(Type::getInt1Ty(M->getContext())),
-          NS->getExportSymbols() /*Is inlined namespace*/);
-      Ops.push_back(ExpConst->getId());
-    }
   }
   if (isNonSemanticDebugInfo())
     transformToConstant(Ops, {LineIdx, ColumnIdx});
@@ -1410,10 +1246,7 @@ SPIRVEntry *LLVMToSPIRVDbgTran::transDebugLoc(const DebugLoc &Loc,
 }
 
 SPIRVEntry *LLVMToSPIRVDbgTran::transDbgInlinedAt(const DILocation *Loc) {
-  // There is a Column operand in NonSemantic.Shader.200 spec
-  if (BM->getDebugInfoEIS() == SPIRVEIS_NonSemantic_Shader_DebugInfo_200)
-    return transDbgInlinedAtNonSemanticShader200(Loc);
-  using namespace SPIRVDebug::Operand::InlinedAt::OpenCL;
+  using namespace SPIRVDebug::Operand::InlinedAt;
   SPIRVWordVec Ops(MinOperandCount);
   Ops[LineIdx] = Loc->getLine();
   Ops[ScopeIdx] = getScope(Loc->getScope())->getId();
@@ -1424,19 +1257,6 @@ SPIRVEntry *LLVMToSPIRVDbgTran::transDbgInlinedAt(const DILocation *Loc) {
   return BM->addDebugInfo(SPIRVDebug::InlinedAt, getVoidTy(), Ops);
 }
 
-SPIRVEntry *LLVMToSPIRVDbgTran::transDbgInlinedAtNonSemanticShader200(
-    const DILocation *Loc) {
-  using namespace SPIRVDebug::Operand::InlinedAt::NonSemantic;
-  SPIRVWordVec Ops(MinOperandCount);
-  Ops[LineIdx] = Loc->getLine();
-  Ops[ColumnIdx] = Loc->getColumn();
-  transformToConstant(Ops, {LineIdx, ColumnIdx});
-  Ops[ScopeIdx] = getScope(Loc->getScope())->getId();
-  if (DILocation *IA = Loc->getInlinedAt())
-    Ops.push_back(transDbgEntry(IA)->getId());
-  return BM->addDebugInfo(SPIRVDebug::InlinedAt, getVoidTy(), Ops);
-}
-
 template <class T>
 SPIRVExtInst *LLVMToSPIRVDbgTran::getSource(const T *DIEntry) {
   const std::string FileName = getFullPath(DIEntry);
@@ -1445,115 +1265,23 @@ SPIRVExtInst *LLVMToSPIRVDbgTran::getSource(const T *DIEntry) {
     return It->second;
 
   using namespace SPIRVDebug::Operand::Source;
-  SPIRVWordVec Ops(MinOperandCount);
+  SPIRVWordVec Ops(OperandCount);
   Ops[FileIdx] = BM->getString(FileName)->getId();
   DIFile *F = DIEntry ? DIEntry->getFile() : nullptr;
-
   if (F && F->getRawChecksum()) {
-    auto CheckSum = F->getChecksum().getValue();
-
-    if (!isNonSemanticDebugInfo())
-      Ops.push_back(BM->getString("//__" + CheckSum.getKindAsString().str() +
-                                  ":" + CheckSum.Value.str())
-                        ->getId());
-    else if (BM->getDebugInfoEIS() ==
-             SPIRVEIS_NonSemantic_Shader_DebugInfo_200) {
-      const SPIRVDebug::FileChecksumKind ChecksumKind =
-          SPIRV::DbgChecksumKindMap::map(CheckSum.Kind);
-
-      Ops.push_back(
-          BM->addIntegerConstant(static_cast<SPIRVTypeInt *>(getInt32Ty()),
-                                 ChecksumKind)
-              ->getId());
-      Ops.push_back(BM->getString(CheckSum.Value.str())->getId());
-    }
+    auto CheckSum = F->getChecksum().value();
+    Ops[TextIdx] = BM->getString("//__" + CheckSum.getKindAsString().str() +
+                                 ":" + CheckSum.Value.str())
+                       ->getId();
+  } else {
+    Ops[TextIdx] = getDebugInfoNone()->getId();
   }
-
-  if (F && F->getRawSource() && isNonSemanticDebugInfo()) {
-    std::string Str = F->getSource().getValue().str();
-    constexpr size_t MaxNumWords =
-        MaxWordCount - 2 /*Fixed WC for SPIRVString*/;
-    constexpr size_t MaxStrSize = MaxNumWords * 4 - 1;
-    const size_t NumWords = getSizeInWords(Str);
-
-    if (BM->getDebugInfoEIS() == SPIRVEIS_NonSemantic_Shader_DebugInfo_200 &&
-        Ops.size() == MinOperandCount) {
-      Ops.push_back(getDebugInfoNoneId());
-      Ops.push_back(getDebugInfoNoneId());
-    }
-    Ops.push_back(BM->getString(Str.substr(0, MaxStrSize))->getId());
-    SPIRVExtInst *Source = static_cast<SPIRVExtInst *>(
-        BM->addDebugInfo(SPIRVDebug::Source, getVoidTy(), Ops));
-    FileMap[FileName] = Source;
-    Str.erase(0, MaxStrSize);
-
-    // No need to generate source continued instructions
-    if (NumWords < MaxNumWords)
-      return Source;
-
-    const uint64_t NumOfContinuedInstructions =
-        NumWords / MaxNumWords - 1 + (NumWords % MaxNumWords ? 1 : 0);
-    for (uint64_t J = 0; J < NumOfContinuedInstructions; J++) {
-      const SPIRVWord Op = BM->getString(Str.substr(0, MaxStrSize))->getId();
-      BM->addDebugInfo(SPIRVDebug::SourceContinued, getVoidTy(), {Op});
-      Str.erase(0, MaxStrSize);
-    }
-    return Source;
-  }
-
   SPIRVExtInst *Source = static_cast<SPIRVExtInst *>(
       BM->addDebugInfo(SPIRVDebug::Source, getVoidTy(), Ops));
   FileMap[FileName] = Source;
   return Source;
 }
 
-void LLVMToSPIRVDbgTran::generateBuildIdentifierAndStoragePath(
-    const DICompileUnit *DIEntry) {
-  // get information from LLVM IR
-  auto BuildIdentifier = DIEntry->getDWOId();
-  const std::string BuildIdentifierString = std::to_string(BuildIdentifier);
-  const std::string StoragePath = DIEntry->getSplitDebugFilename().str();
-
-  using namespace SPIRVDebug::Operand;
-
-  if (BuildIdentifierInsn || StoragePathInsn) {
-#ifndef NDEBUG
-    assert(BuildIdentifierInsn && StoragePathInsn &&
-           "BuildIdentifier and StoragePath instructions must both be created");
-
-    auto PreviousBuildIdentifierString =
-        BM->get<SPIRVString>(
-              BuildIdentifierInsn
-                  ->getArguments()[BuildIdentifier::IdentifierIdx])
-            ->getStr();
-    assert(PreviousBuildIdentifierString == BuildIdentifierString &&
-           "New BuildIdentifier should match previous BuildIdentifier");
-    auto PreviousStoragePath =
-        BM->get<SPIRVString>(
-              StoragePathInsn->getArguments()[StoragePath::PathIdx])
-            ->getStr();
-    assert(PreviousStoragePath == StoragePath &&
-           "New StoragePath should match previous StoragePath");
-#endif
-    return;
-  }
-
-  // generate BuildIdentifier inst
-  SPIRVWordVec BuildIdentifierOps(BuildIdentifier::OperandCount);
-  BuildIdentifierOps[BuildIdentifier::IdentifierIdx] =
-      BM->getString(BuildIdentifierString)->getId();
-  BuildIdentifierOps[BuildIdentifier::FlagsIdx] =
-      BM->getLiteralAsConstant(1)->getId(); // Placeholder value for now
-  BuildIdentifierInsn = static_cast<SPIRVExtInst *>(BM->addDebugInfo(
-      SPIRVDebug::BuildIdentifier, getVoidTy(), BuildIdentifierOps));
-
-  // generate StoragePath inst
-  SPIRVWordVec StoragePathOps(StoragePath::OperandCount);
-  StoragePathOps[StoragePath::PathIdx] = BM->getString(StoragePath)->getId();
-  StoragePathInsn = static_cast<SPIRVExtInst *>(
-      BM->addDebugInfo(SPIRVDebug::StoragePath, getVoidTy(), StoragePathOps));
-}
-
 SPIRVEntry *LLVMToSPIRVDbgTran::transDbgFileType(const DIFile *F) {
   return BM->getString(getFullPath(F));
 }
@@ -1591,9 +1319,7 @@ SPIRVEntry *LLVMToSPIRVDbgTran::transDbgExpression(const DIExpression *Expr) {
         SPIRV::DbgExpressionOpCodeMap::map(DWARFOpCode);
     if (OpCountMap.find(OC) == OpCountMap.end())
       report_fatal_error(llvm::Twine("unknown opcode found in DIExpression"));
-    if (OC > SPIRVDebug::Fragment &&
-        !(BM->allowExtraDIExpressions() ||
-          BM->getDebugInfoEIS() == SPIRVEIS_NonSemantic_Shader_DebugInfo_200))
+    if (OC > SPIRVDebug::Fragment && !BM->allowExtraDIExpressions())
       report_fatal_error(
           llvm::Twine("unsupported opcode found in DIExpression"));
 
@@ -1602,11 +1328,8 @@ SPIRVEntry *LLVMToSPIRVDbgTran::transDbgExpression(const DIExpression *Expr) {
     Op[OpCodeIdx] = OC;
     if (isNonSemanticDebugInfo())
       transformToConstant(Op, {OpCodeIdx});
-    for (unsigned J = 1; J < OpCount; ++J) {
+    for (unsigned J = 1; J < OpCount; ++J)
       Op[J] = Expr->getElement(++I);
-      if (isNonSemanticDebugInfo())
-        transformToConstant(Op, {J});
-    }
     auto *Operation = BM->addDebugInfo(SPIRVDebug::Operation, getVoidTy(), Op);
     Operations.push_back(Operation->getId());
   }
@@ -1618,23 +1341,17 @@ SPIRVEntry *LLVMToSPIRVDbgTran::transDbgExpression(const DIExpression *Expr) {
 SPIRVEntry *
 LLVMToSPIRVDbgTran::transDbgImportedEntry(const DIImportedEntity *IE) {
   using namespace SPIRVDebug::Operand::ImportedEntity;
+  SPIRVWordVec Ops(OperandCount);
   auto Tag = static_cast<dwarf::Tag>(IE->getTag());
-  // FIXME: 'OpenCL/bugged' version is kept because it's hard to remove it
-  // It's W/A for missing 2nd index in OpenCL's implementation
-  const SPIRVWord OffsetIdx =
-      isNonSemanticDebugInfo() ? OperandCount - NonSemantic::OperandCount : 0;
-  SPIRVWordVec Ops(OperandCount - OffsetIdx);
   Ops[NameIdx] = BM->getString(IE->getName().str())->getId();
   Ops[TagIdx] = SPIRV::DbgImportedEntityMap::map(Tag);
-  Ops[SourceIdx - OffsetIdx] = getSource(IE->getFile())->getId();
-  Ops[EntityIdx - OffsetIdx] = transDbgEntry(IE->getEntity())->getId();
-  Ops[LineIdx - OffsetIdx] = IE->getLine();
-  // This version of DIImportedEntity has no column number
-  Ops[ColumnIdx - OffsetIdx] = 0;
-  Ops[ParentIdx - OffsetIdx] = getScope(IE->getScope())->getId();
+  Ops[SourceIdx] = getSource(IE->getFile())->getId();
+  Ops[EntityIdx] = transDbgEntry(IE->getEntity())->getId();
+  Ops[LineIdx] = IE->getLine();
+  Ops[ColumnIdx] = 0; // This version of DIImportedEntity has no column number
+  Ops[ParentIdx] = getScope(IE->getScope())->getId();
   if (isNonSemanticDebugInfo())
-    transformToConstant(Ops,
-                        {TagIdx, LineIdx - OffsetIdx, ColumnIdx - OffsetIdx});
+    transformToConstant(Ops, {TagIdx, LineIdx, ColumnIdx});
   return BM->addDebugInfo(SPIRVDebug::ImportedEntity, getVoidTy(), Ops);
 }
 
