@@ -175,7 +175,7 @@ void SPIRVEntry::encodeLine(spv_ostream &O) const {
   if (!Module)
     return;
   const std::shared_ptr<const SPIRVLine> &CurrLine = Module->getCurrentLine();
-  if (Line && (!CurrLine || *Line != *CurrLine)) {
+  if (Line && ((CurrLine && *Line != *CurrLine) || !CurrLine)) {
     O << *Line;
     Module->setCurrentLine(Line);
   }
@@ -183,43 +183,8 @@ void SPIRVEntry::encodeLine(spv_ostream &O) const {
     Module->setCurrentLine(nullptr);
 }
 
-namespace {
-bool isDebugLineEqual(const SPIRVExtInst &DL1, const SPIRVExtInst &DL2) {
-  std::vector<SPIRVWord> DL1Args = DL1.getArguments();
-  std::vector<SPIRVWord> DL2Args = DL2.getArguments();
-
-  using namespace SPIRVDebug::Operand::DebugLine;
-  assert(DL1Args.size() == OperandCount && DL2Args.size() == OperandCount &&
-         "Invalid number of operands");
-  return DL1Args[SourceIdx] == DL2Args[SourceIdx] &&
-         DL1Args[StartIdx] == DL2Args[StartIdx] &&
-         DL1Args[EndIdx] == DL2Args[EndIdx] &&
-         DL1Args[ColumnStartIdx] == DL2Args[ColumnStartIdx] &&
-         DL1Args[ColumnEndIdx] == DL2Args[ColumnEndIdx];
-}
-} // namespace
-
-void SPIRVEntry::encodeDebugLine(spv_ostream &O) const {
-  if (!Module)
-    return;
-  const std::shared_ptr<const SPIRVExtInst> &CurrDebugLine =
-      Module->getCurrentDebugLine();
-  if (DebugLine &&
-      (!CurrDebugLine || !isDebugLineEqual(*DebugLine, *CurrDebugLine))) {
-    O << *DebugLine;
-    Module->setCurrentDebugLine(DebugLine);
-  }
-  if (isEndOfBlock() ||
-      isExtInst(SPIRVEIS_NonSemantic_Shader_DebugInfo_100,
-                SPIRVDebug::DebugNoLine) ||
-      isExtInst(SPIRVEIS_NonSemantic_Shader_DebugInfo_200,
-                SPIRVDebug::DebugNoLine))
-    Module->setCurrentDebugLine(nullptr);
-}
-
 void SPIRVEntry::encodeAll(spv_ostream &O) const {
   encodeLine(O);
-  encodeDebugLine(O);
   encodeWordCountOpCode(O);
   encode(O);
   encodeChildren(O);
@@ -336,15 +301,10 @@ void SPIRVEntry::setLine(const std::shared_ptr<const SPIRVLine> &L) {
   SPIRVDBG(if (L) spvdbgs() << "[setLine] " << *L << '\n';)
 }
 
-void SPIRVEntry::setDebugLine(const std::shared_ptr<const SPIRVExtInst> &DL) {
-  DebugLine = DL;
-  SPIRVDBG(if (DL) spvdbgs() << "[setDebugLine] " << *DL << '\n';)
-}
-
 void SPIRVEntry::addMemberDecorate(SPIRVMemberDecorate *Dec) {
-  assert(canHaveMemberDecorates() &&
-         MemberDecorates.find(Dec->getPair()) == MemberDecorates.end());
-  MemberDecorates[Dec->getPair()] = Dec;
+  assert(canHaveMemberDecorates());
+  MemberDecorates.insert(std::make_pair(
+      std::make_pair(Dec->getMemberNumber(), Dec->getDecorateKind()), Dec));
   Module->addDecorate(Dec);
   SPIRVDBG(spvdbgs() << "[addMemberDecorate] " << *Dec << '\n';)
 }
@@ -440,6 +400,33 @@ SPIRVEntry::getMemberDecorationStringLiteral(Decoration Kind,
   return getVecString(Loc->second->getVecLiteral());
 }
 
+std::vector<std::vector<std::string>>
+SPIRVEntry::getAllDecorationStringLiterals(Decoration Kind) const {
+  auto Loc = Decorates.find(Kind);
+  if (Loc == Decorates.end())
+    return {};
+
+  std::vector<std::vector<std::string>> Literals;
+  auto It = Decorates.equal_range(Kind);
+  for (auto Itr = It.first; Itr != It.second; ++Itr)
+    Literals.push_back(getVecString(Itr->second->getVecLiteral()));
+  return Literals;
+}
+
+std::vector<std::vector<std::string>>
+SPIRVEntry::getAllMemberDecorationStringLiterals(Decoration Kind,
+                                                 SPIRVWord MemberNumber) const {
+  auto Loc = MemberDecorates.find({MemberNumber, Kind});
+  if (Loc == MemberDecorates.end())
+    return {};
+
+  std::vector<std::vector<std::string>> Literals;
+  auto It = MemberDecorates.equal_range({MemberNumber, Kind});
+  for (auto Itr = It.first; Itr != It.second; ++Itr)
+    Literals.push_back(getVecString(Itr->second->getVecLiteral()));
+  return Literals;
+}
+
 std::vector<SPIRVWord>
 SPIRVEntry::getDecorationLiterals(Decoration Kind) const {
   auto Loc = Decorates.find(Kind);
@@ -615,7 +602,9 @@ void SPIRVExecutionMode::decode(std::istream &I) {
   getDecoder(I) >> Target >> ExecMode;
   switch (static_cast<uint32_t>(ExecMode)) {
   case ExecutionModeLocalSize:
+  case ExecutionModeLocalSizeId:
   case ExecutionModeLocalSizeHint:
+  case ExecutionModeLocalSizeHintId:
   case ExecutionModeMaxWorkgroupSizeINTEL:
     WordLiterals.resize(3);
     break;
@@ -634,10 +623,12 @@ void SPIRVExecutionMode::decode(std::istream &I) {
   case ExecutionModeSharedLocalMemorySizeINTEL:
   case ExecutionModeNamedBarrierCountINTEL:
   case ExecutionModeSubgroupSize:
+  case ExecutionModeSubgroupsPerWorkgroup:
+  case ExecutionModeSubgroupsPerWorkgroupId:
   case ExecutionModeMaxWorkDimINTEL:
   case ExecutionModeNumSIMDWorkitemsINTEL:
   case ExecutionModeSchedulerTargetFmaxMhzINTEL:
-  case internal::ExecutionModeStreamingInterfaceINTEL:
+  case ExecutionModeStreamingInterfaceINTEL:
   case internal::ExecutionModeMaximumRegistersINTEL:
   case internal::ExecutionModeMaximumRegistersIdINTEL:
   case internal::ExecutionModeNamedMaximumRegistersINTEL:
@@ -685,6 +676,8 @@ void SPIRVLine::encode(spv_ostream &O) const {
 
 void SPIRVLine::decode(std::istream &I) {
   getDecoder(I) >> FileName >> Line >> Column;
+  std::shared_ptr<const SPIRVLine> L(this);
+  Module->setCurrentLine(L);
 }
 
 void SPIRVLine::validate() const {
