@@ -71,7 +71,7 @@ inline bool isValid(spv::ExecutionModel V) {
   case ExecutionModelClosestHitKHR:
   case ExecutionModelMissKHR:
   case ExecutionModelCallableKHR:
-  case internal::ExecutionModeStreamingInterfaceINTEL:
+  case ExecutionModeStreamingInterfaceINTEL:
   case internal::ExecutionModeMaximumRegistersINTEL:
   case internal::ExecutionModeMaximumRegistersIdINTEL:
   case internal::ExecutionModeNamedMaximumRegistersINTEL:
