@@ -80,6 +80,9 @@ std::string to_string(uint32_t Version) {
   case static_cast<uint32_t>(VersionNumber::SPIRV_1_5):
     Res = "1.5";
     break;
+  case static_cast<uint32_t>(VersionNumber::SPIRV_1_6):
+    Res = "1.6";
+    break;
   default:
     Res = "unknown";
   }
@@ -165,6 +168,13 @@ public:
   SPIRVConstant *getLiteralAsConstant(unsigned Literal) override;
   unsigned getNumFunctions() const override { return FuncVec.size(); }
   unsigned getNumVariables() const override { return VariableVec.size(); }
+  std::vector<SPIRVValue *> getFunctionPointers() const override {
+    std::vector<SPIRVValue *> Res;
+    for (auto *C : ConstVec)
+      if (C->getOpCode() == OpConstantFunctionPointerINTEL)
+        Res.emplace_back(C);
+    return Res;
+  }
   SourceLanguage getSourceLanguage(SPIRVWord *Ver = nullptr) const override {
     if (Ver)
       *Ver = SrcLangVer;
@@ -233,13 +243,6 @@ public:
                SPIRVWord Column) override;
   const std::shared_ptr<const SPIRVLine> &getCurrentLine() const override;
   void setCurrentLine(const std::shared_ptr<const SPIRVLine> &Line) override;
-  void addDebugLine(SPIRVEntry *E, SPIRVType *TheType, SPIRVId FileNameId,
-                    SPIRVWord LineStart, SPIRVWord LineEnd,
-                    SPIRVWord ColumnStart, SPIRVWord ColumnEnd) override;
-  const std::shared_ptr<const SPIRVExtInst> &
-  getCurrentDebugLine() const override;
-  void setCurrentDebugLine(
-      const std::shared_ptr<const SPIRVExtInst> &DebugLine) override;
   void addCapability(SPIRVCapabilityKind) override;
   void addCapabilityInternal(SPIRVCapabilityKind) override;
   void addExtension(ExtensionID) override;
@@ -290,6 +293,9 @@ public:
   addJointMatrixINTELType(SPIRVType *, std::vector<SPIRVValue *>) override;
   SPIRVTypeCooperativeMatrixKHR *
   addCooperativeMatrixKHRType(SPIRVType *, std::vector<SPIRVValue *>) override;
+  SPIRVInstruction *
+  addCooperativeMatrixLengthKHRInst(SPIRVType *, SPIRVType *,
+                                    SPIRVBasicBlock *) override;
   SPIRVType *addOpaqueGenericType(Op) override;
   SPIRVTypeDeviceEvent *addDeviceEventType() override;
   SPIRVTypeQueue *addQueueType() override;
@@ -349,8 +355,6 @@ public:
                                const std::vector<SPIRVValue *> &,
                                SPIRVBasicBlock *,
                                SPIRVInstruction * = nullptr) override;
-  SPIRVEntry *createDebugInfo(SPIRVWord, SPIRVType *TheType,
-                              const std::vector<SPIRVWord> &) override;
   SPIRVEntry *addDebugInfo(SPIRVWord, SPIRVType *TheType,
                            const std::vector<SPIRVWord> &) override;
   SPIRVEntry *addAuxData(SPIRVWord, SPIRVType *TheType,
@@ -566,7 +570,6 @@ private:
   SPIRVStringVec StringVec;
   SPIRVMemberNameVec MemberNameVec;
   std::shared_ptr<const SPIRVLine> CurrentLine;
-  std::shared_ptr<const SPIRVExtInst> CurrentDebugLine;
   SPIRVDecorateVec DecorateVec;
   SPIRVDecGroupVec DecGroupVec;
   SPIRVGroupDecVec GroupDecVec;
@@ -620,60 +623,6 @@ void SPIRVModuleImpl::addLine(SPIRVEntry *E, SPIRVId FileNameId, SPIRVWord Line,
   E->setLine(CurrentLine);
 }
 
-const std::shared_ptr<const SPIRVExtInst> &
-SPIRVModuleImpl::getCurrentDebugLine() const {
-  return CurrentDebugLine;
-}
-
-void SPIRVModuleImpl::setCurrentDebugLine(
-    const std::shared_ptr<const SPIRVExtInst> &DebugLine) {
-  CurrentDebugLine = DebugLine;
-}
-
-namespace {
-bool isDebugLineEqual(const SPIRVExtInst &CurrentDebugLine, SPIRVId FileNameId,
-                      SPIRVId LineStartId, SPIRVId LineEndId,
-                      SPIRVId ColumnStartId, SPIRVId ColumnEndId) {
-  assert(CurrentDebugLine.getExtOp() == SPIRVDebug::DebugLine);
-  const std::vector<SPIRVWord> CurrentDebugLineArgs =
-      CurrentDebugLine.getArguments();
-
-  using namespace SPIRVDebug::Operand::DebugLine;
-  return CurrentDebugLineArgs[SourceIdx] == FileNameId &&
-         CurrentDebugLineArgs[StartIdx] == LineStartId &&
-         CurrentDebugLineArgs[EndIdx] == LineEndId &&
-         CurrentDebugLineArgs[ColumnStartIdx] == ColumnStartId &&
-         CurrentDebugLineArgs[ColumnEndIdx] == ColumnEndId;
-}
-} // namespace
-
-void SPIRVModuleImpl::addDebugLine(SPIRVEntry *E, SPIRVType *TheType,
-                                   SPIRVId FileNameId, SPIRVWord LineStart,
-                                   SPIRVWord LineEnd, SPIRVWord ColumnStart,
-                                   SPIRVWord ColumnEnd) {
-  if (!(CurrentDebugLine &&
-        isDebugLineEqual(*CurrentDebugLine, FileNameId,
-                         getLiteralAsConstant(LineStart)->getId(),
-                         getLiteralAsConstant(LineEnd)->getId(),
-                         getLiteralAsConstant(ColumnStart)->getId(),
-                         getLiteralAsConstant(ColumnEnd)->getId()))) {
-    using namespace SPIRVDebug::Operand::DebugLine;
-
-    std::vector<SPIRVWord> DebugLineOps(OperandCount);
-    DebugLineOps[SourceIdx] = FileNameId;
-    DebugLineOps[StartIdx] = getLiteralAsConstant(LineStart)->getId();
-    DebugLineOps[EndIdx] = getLiteralAsConstant(LineEnd)->getId();
-    DebugLineOps[ColumnStartIdx] = getLiteralAsConstant(ColumnStart)->getId();
-    DebugLineOps[ColumnEndIdx] = getLiteralAsConstant(ColumnEnd)->getId();
-
-    CurrentDebugLine.reset(static_cast<SPIRVExtInst *>(
-        createDebugInfo(SPIRVDebug::DebugLine, TheType, DebugLineOps)));
-  }
-
-  assert(E && "invalid entry");
-  E->setDebugLine(CurrentDebugLine);
-}
-
 SPIRVValue *SPIRVModuleImpl::addSamplerConstant(SPIRVType *TheType,
                                                 SPIRVWord AddrMode,
                                                 SPIRVWord ParametricMode,
@@ -724,8 +673,8 @@ void SPIRVModuleImpl::addCapability(SPIRVCapabilityKind Cap) {
     // While we are reading existing SPIR-V we need to read it as-is and don't
     // add required extensions for each entry automatically
     auto Ext = CapObj->getRequiredExtension();
-    if (Ext.hasValue())
-      addExtension(Ext.getValue());
+    if (Ext.has_value())
+      addExtension(Ext.value());
   }
 
   CapMap.insert(std::make_pair(Cap, CapObj));
@@ -853,8 +802,8 @@ SPIRVEntry *SPIRVModuleImpl::addEntry(SPIRVEntry *Entry) {
     // While we are reading existing SPIR-V we need to read it as-is and don't
     // add required extensions for each entry automatically
     auto Ext = Entry->getRequiredExtension();
-    if (Ext.hasValue())
-      addExtension(Ext.getValue());
+    if (Ext.has_value())
+      addExtension(Ext.value());
   }
 
   return Entry;
@@ -1046,6 +995,14 @@ SPIRVModuleImpl::addCooperativeMatrixKHRType(SPIRVType *CompType,
       new SPIRVTypeCooperativeMatrixKHR(this, getId(), CompType, Args));
 }
 
+SPIRVInstruction *SPIRVModuleImpl::addCooperativeMatrixLengthKHRInst(
+    SPIRVType *RetTy, SPIRVType *MatTy, SPIRVBasicBlock *BB) {
+  return addInstruction(
+      SPIRVInstTemplateBase::create(OpCooperativeMatrixLengthKHR, RetTy,
+                                    getId(), getVec(MatTy->getId()), BB, this),
+      BB);
+}
+
 SPIRVType *SPIRVModuleImpl::addOpaqueGenericType(Op TheOpCode) {
   return addType(new SPIRVTypeOpaqueGeneric(TheOpCode, this, getId()));
 }
@@ -1160,17 +1117,19 @@ SPIRVEntry *SPIRVModuleImpl::replaceForward(SPIRVForward *Forward,
                                             SPIRVEntry *Entry) {
   SPIRVId Id = Entry->getId();
   SPIRVId ForwardId = Forward->getId();
-  if (ForwardId == Id)
+  if (ForwardId == Id) {
     IdEntryMap[Id] = Entry;
-  else {
+    // Annotations include name, decorations, execution modes
+    Entry->takeAnnotations(Forward);
+  } else {
     auto Loc = IdEntryMap.find(Id);
     assert(Loc != IdEntryMap.end());
     IdEntryMap.erase(Loc);
     Entry->setId(ForwardId);
     IdEntryMap[ForwardId] = Entry;
+    // Replace current Id with ForwardId in decorates.
+    Entry->replaceTargetIdInDecorates(ForwardId);
   }
-  // Annotations include name, decorations, execution modes
-  Entry->takeAnnotations(Forward);
   delete Forward;
   return Entry;
 }
@@ -1434,16 +1393,11 @@ SPIRVInstruction *SPIRVModuleImpl::addExtInst(
       InsertBefore);
 }
 
-SPIRVEntry *
-SPIRVModuleImpl::createDebugInfo(SPIRVWord InstId, SPIRVType *TheType,
-                                 const std::vector<SPIRVWord> &Args) {
-  return new SPIRVExtInst(this, getId(), TheType, SPIRVEIS_OpenCL_DebugInfo_100,
-                          ExtInstSetIds[getDebugInfoEIS()], InstId, Args);
-}
-
 SPIRVEntry *SPIRVModuleImpl::addDebugInfo(SPIRVWord InstId, SPIRVType *TheType,
                                           const std::vector<SPIRVWord> &Args) {
-  return addEntry(createDebugInfo(InstId, TheType, Args));
+  return addEntry(
+      new SPIRVExtInst(this, getId(), TheType, SPIRVEIS_OpenCL_DebugInfo_100,
+                       ExtInstSetIds[getDebugInfoEIS()], InstId, Args));
 }
 
 SPIRVEntry *SPIRVModuleImpl::addAuxData(SPIRVWord InstId, SPIRVType *TheType,
@@ -1780,9 +1734,8 @@ SPIRVInstruction *SPIRVModuleImpl::addExpectKHRInst(SPIRVType *ResultTy,
 // Create AliasDomainDeclINTEL/AliasScopeDeclINTEL/AliasScopeListDeclINTEL
 // instructions
 template <typename AliasingInstType>
-SPIRVEntry *
-SPIRVModuleImpl::getOrAddMemAliasingINTELInst(std::vector<SPIRVId> Args,
-                                              llvm::MDNode *MD) {
+SPIRVEntry *SPIRVModuleImpl::getOrAddMemAliasingINTELInst(
+    std::vector<SPIRVId> Args, llvm::MDNode *MD) {
   assert(MD && "noalias/alias.scope metadata can't be null");
   // Don't duplicate aliasing instruction. For that use a map with a MDNode key
   if (AliasInstMDMap.find(MD) != AliasInstMDMap.end())
@@ -1793,23 +1746,20 @@ SPIRVModuleImpl::getOrAddMemAliasingINTELInst(std::vector<SPIRVId> Args,
 }
 
 // Create AliasDomainDeclINTEL instruction
-SPIRVEntry *
-SPIRVModuleImpl::getOrAddAliasDomainDeclINTELInst(std::vector<SPIRVId> Args,
-                                                  llvm::MDNode *MD) {
+SPIRVEntry *SPIRVModuleImpl::getOrAddAliasDomainDeclINTELInst(
+    std::vector<SPIRVId> Args, llvm::MDNode *MD) {
   return getOrAddMemAliasingINTELInst<SPIRVAliasDomainDeclINTEL>(Args, MD);
 }
 
 // Create AliasScopeDeclINTEL instruction
-SPIRVEntry *
-SPIRVModuleImpl::getOrAddAliasScopeDeclINTELInst(std::vector<SPIRVId> Args,
-                                                 llvm::MDNode *MD) {
+SPIRVEntry *SPIRVModuleImpl::getOrAddAliasScopeDeclINTELInst(
+    std::vector<SPIRVId> Args, llvm::MDNode *MD) {
   return getOrAddMemAliasingINTELInst<SPIRVAliasScopeDeclINTEL>(Args, MD);
 }
 
 // Create AliasScopeListDeclINTEL instruction
-SPIRVEntry *
-SPIRVModuleImpl::getOrAddAliasScopeListDeclINTELInst(std::vector<SPIRVId> Args,
-                                                     llvm::MDNode *MD) {
+SPIRVEntry *SPIRVModuleImpl::getOrAddAliasScopeListDeclINTELInst(
+    std::vector<SPIRVId> Args, llvm::MDNode *MD) {
   return getOrAddMemAliasingINTELInst<SPIRVAliasScopeListDeclINTEL>(Args, MD);
 }
 
@@ -1954,7 +1904,7 @@ public:
     for (auto *V : VariableVec)
       EntryStateMap[V] = DFSState::Unvisited;
     // Run topoligical sort
-    for (auto ES : EntryStateMap) {
+    for (const auto &ES : EntryStateMap) {
       if (visit(ES.first))
         llvm_unreachable("Cyclic dependency for types detected");
     }
@@ -1973,7 +1923,6 @@ spv_ostream &operator<<(spv_ostream &O, SPIRVModule &M) {
   SPIRVModuleImpl &MI = *static_cast<SPIRVModuleImpl *>(&M);
   // Start tracking of the current line with no line
   MI.CurrentLine.reset();
-  MI.CurrentDebugLine.reset();
 
   SPIRVEncoder Encoder(O);
   Encoder << MagicNumber << MI.SPIRVVersion
@@ -2022,7 +1971,7 @@ spv_ostream &operator<<(spv_ostream &O, SPIRVModule &M) {
   }
 
   if (M.isAllowedToUseExtension(
-          ExtensionID::SPV_INTEL_memory_access_aliasing)) {
+        ExtensionID::SPV_INTEL_memory_access_aliasing)) {
     O << SPIRVNL() << MI.AliasInstMDVec;
   }
 
@@ -2036,17 +1985,6 @@ spv_ostream &operator<<(spv_ostream &O, SPIRVModule &M) {
     O << SPIRVNL() << MI.AsmTargetVec << MI.AsmVec;
   }
 
-  // At this point we know that FunctionDefinition could have been included both
-  // into DebugInstVec and into basick block of function from FuncVec.
-  // By spec we should only have this instruction to be present inside the
-  // function body, so removing it from the DebugInstVec to avoid duplication.
-  MI.DebugInstVec.erase(
-      std::remove_if(MI.DebugInstVec.begin(), MI.DebugInstVec.end(),
-                     [](SPIRVExtInst *I) {
-                       return I->getExtOp() == SPIRVDebug::FunctionDefinition;
-                     }),
-      MI.DebugInstVec.end());
-
   O << SPIRVNL() << MI.DebugInstVec << MI.AuxDataInstVec << SPIRVNL()
     << MI.FuncVec;
   return O;
@@ -2133,9 +2071,7 @@ std::istream &operator>>(std::istream &I, SPIRVModule &M) {
   }
 
   Decoder >> MI.SPIRVVersion;
-  bool SPIRVVersionIsKnown =
-      static_cast<uint32_t>(VersionNumber::MinimumVersion) <= MI.SPIRVVersion &&
-      MI.SPIRVVersion <= static_cast<uint32_t>(VersionNumber::MaximumVersion);
+  bool SPIRVVersionIsKnown = isSPIRVVersionKnown(MI.SPIRVVersion);
   if (!M.getErrorLog().checkError(
           SPIRVVersionIsKnown, SPIRVEC_InvalidModule,
           "unsupported SPIR-V version number '" + to_string(MI.SPIRVVersion) +
